@@ -3,7 +3,7 @@ package Net::DNS::Header;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: Header.pm 1899 2023-02-20 15:34:02Z willem $)[2];
+our $VERSION = (qw$Id: Header.pm 2002 2025-01-07 09:57:46Z willem $)[2];
 
 
 =head1 NAME
@@ -12,10 +12,10 @@ Net::DNS::Header - DNS packet header
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
+	use Net::DNS;
 
-    $packet = Net::DNS::Packet->new();
-    $header = $packet->header;
+	$packet = Net::DNS::Packet->new();
+	$header = $packet->header;
 
 
 =head1 DESCRIPTION
@@ -36,8 +36,8 @@ use Net::DNS::Parameters qw(:opcode :rcode);
 
 =head2 $packet->header
 
-    $packet = Net::DNS::Packet->new();
-    $header = $packet->header;
+	$packet = Net::DNS::Packet->new();
+	$header = $packet->header;
 
 Net::DNS::Header objects emanate from the Net::DNS::Packet header()
 method, and contain an opaque reference to the parent Packet object.
@@ -49,7 +49,7 @@ structures.
 
 =head2 string
 
-    print $packet->header->string;
+	print $packet->header->string;
 
 Returns a string representation of the packet header.
 
@@ -66,10 +66,16 @@ sub string {
 	my $an	   = $self->ancount;
 	my $ns	   = $self->nscount;
 	my $ar	   = $self->arcount;
+	my $dispid = defined $id ? $id : 'undef';
+	return <<"QQ" if $opcode eq 'DSO';
+;;	id = $dispid	qr = $qr
+;;	opcode = $opcode	rcode = $rcode
+QQ
 	return <<"QQ" if $opcode eq 'UPDATE';
-;;	id = $id
-;;	qr = $qr		opcode = $opcode	rcode = $rcode
-;;	zocount = $qd	prcount = $an	upcount = $ns	adcount = $ar
+;;	id = $dispid	qr = $qr
+;;	opcode = $opcode	rcode = $rcode
+;;	zocount = $qd	prcount = $an
+;;	upcount = $ns	adcount = $ar
 QQ
 	my $aa = $self->aa;
 	my $tc = $self->tc;
@@ -79,19 +85,21 @@ QQ
 	my $ad = $self->ad;
 	my $cd = $self->cd;
 	my $do = $self->do;
+	my $co = $self->co;
 	return <<"QQ";
-;;	id = $id
+;;	id = $dispid
 ;;	qr = $qr	aa = $aa	tc = $tc	rd = $rd	opcode = $opcode
 ;;	ra = $ra	z  = $zz	ad = $ad	cd = $cd	rcode  = $rcode
-;;	qdcount = $qd	ancount = $an	nscount = $ns	arcount = $ar
-;;	do = $do
+;;	do = $do	co = $co
+;;	qdcount = $qd	ancount = $an
+;;	nscount = $ns	arcount = $ar
 QQ
 }
 
 
 =head2 print
 
-    $packet->header->print;
+	$packet->header->print;
 
 Prints the string representation of the packet header.
 
@@ -105,34 +113,24 @@ sub print {
 
 =head2 id
 
-    print "query id = ", $packet->header->id, "\n";
-    $packet->header->id(1234);
+	print "query id = ", $packet->header->id, "\n";
+	$packet->header->id(1234);
 
 Gets or sets the query identification number.
 
-A random value is assigned if the argument value is undefined.
-
 =cut
 
-my ( $cache1, $cache2, $limit );				# two layer cache
-
 sub id {
 	my ( $self, @value ) = @_;
 	for (@value) { $$self->{id} = $_ }
-	my $ident = $$self->{id};
-	return $ident if $ident;
-	return $ident if defined($ident) && $self->opcode eq 'DSO';
-	( $cache1, $cache2, $limit ) = ( {0 => 1}, $cache1, 50 ) unless $limit--;
-	$ident = int rand(0xffff);				# preserve short-term uniqueness
-	$ident = int rand(0xffff) while $cache1->{$ident}++ + exists( $cache2->{$ident} );
-	return $$self->{id} = $ident;
+	return $$self->{id};
 }
 
 
 =head2 opcode
 
-    print "query opcode = ", $packet->header->opcode, "\n";
-    $packet->header->opcode("UPDATE");
+	print "query opcode = ", $packet->header->opcode, "\n";
+	$packet->header->opcode("UPDATE");
 
 Gets or sets the query opcode (the purpose of the query).
 
@@ -152,8 +150,8 @@ sub opcode {
 
 =head2 rcode
 
-    print "query response code = ", $packet->header->rcode, "\n";
-    $packet->header->rcode("SERVFAIL");
+	print "query response code = ", $packet->header->rcode, "\n";
+	$packet->header->rcode("SERVFAIL");
 
 Gets or sets the query response code (the status of the query).
 
@@ -180,8 +178,8 @@ sub rcode {
 
 =head2 qr
 
-    print "query response flag = ", $packet->header->qr, "\n";
-    $packet->header->qr(0);
+	print "query response flag = ", $packet->header->qr, "\n";
+	$packet->header->qr(0);
 
 Gets or sets the query response flag.
 
@@ -195,8 +193,8 @@ sub qr {
 
 =head2 aa
 
-    print "response is ", $packet->header->aa ? "" : "non-", "authoritative\n";
-    $packet->header->aa(0);
+	print "response is ", $packet->header->aa ? "" : "non-", "authoritative\n";
+	$packet->header->aa(0);
 
 Gets or sets the authoritative answer flag.
 
@@ -210,8 +208,8 @@ sub aa {
 
 =head2 tc
 
-    print "packet is ", $packet->header->tc ? "" : "not ", "truncated\n";
-    $packet->header->tc(0);
+	print "packet is ", $packet->header->tc ? "" : "not ", "truncated\n";
+	$packet->header->tc(0);
 
 Gets or sets the truncated packet flag.
 
@@ -225,8 +223,8 @@ sub tc {
 
 =head2 rd
 
-    print "recursion was ", $packet->header->rd ? "" : "not ", "desired\n";
-    $packet->header->rd(0);
+	print "recursion was ", $packet->header->rd ? "" : "not ", "desired\n";
+	$packet->header->rd(0);
 
 Gets or sets the recursion desired flag.
 
@@ -240,8 +238,8 @@ sub rd {
 
 =head2 ra
 
-    print "recursion is ", $packet->header->ra ? "" : "not ", "available\n";
-    $packet->header->ra(0);
+	print "recursion is ", $packet->header->ra ? "" : "not ", "available\n";
+	$packet->header->ra(0);
 
 Gets or sets the recursion available flag.
 
@@ -267,7 +265,7 @@ sub z {
 
 =head2 ad
 
-    print "The response has ", $packet->header->ad ? "" : "not", "been verified\n";
+	print "The response has ", $packet->header->ad ? "" : "not", "been verified\n";
 
 Relevant in DNSSEC context.
 
@@ -285,8 +283,8 @@ sub ad {
 
 =head2 cd
 
-    print "checking was ", $packet->header->cd ? "not" : "", "desired\n";
-    $packet->header->cd(0);
+	print "checking was ", $packet->header->cd ? "not" : "", "desired\n";
+	$packet->header->cd(0);
 
 Gets or sets the checking disabled flag.
 
@@ -300,7 +298,7 @@ sub cd {
 
 =head2 qdcount, zocount
 
-    print "# of question records: ", $packet->header->qdcount, "\n";
+	print "# of question records: ", $packet->header->qdcount, "\n";
 
 Returns the number of records in the question section of the packet.
 In dynamic update packets, this field is known as C<zocount> and refers
@@ -308,18 +306,16 @@ to the number of RRs in the zone section.
 
 =cut
 
-our $warned;
-
 sub qdcount {
 	my ( $self, @value ) = @_;
-	for (@value) { carp 'packet->header->qdcount attribute is read-only' unless $warned++ }
+	for (@value) { $self->_warn('packet->header->qdcount is read-only') }
 	return $$self->{count}[0] || scalar @{$$self->{question}};
 }
 
 
 =head2 ancount, prcount
 
-    print "# of answer records: ", $packet->header->ancount, "\n";
+	print "# of answer records: ", $packet->header->ancount, "\n";
 
 Returns the number of records in the answer section of the packet
 which may, in the case of corrupt packets, differ from the actual
@@ -331,14 +327,14 @@ to the number of RRs in the prerequisite section.
 
 sub ancount {
 	my ( $self, @value ) = @_;
-	for (@value) { carp 'packet->header->ancount attribute is read-only' unless $warned++ }
+	for (@value) { $self->_warn('packet->header->ancount is read-only') }
 	return $$self->{count}[1] || scalar @{$$self->{answer}};
 }
 
 
 =head2 nscount, upcount
 
-    print "# of authority records: ", $packet->header->nscount, "\n";
+	print "# of authority records: ", $packet->header->nscount, "\n";
 
 Returns the number of records in the authority section of the packet
 which may, in the case of corrupt packets, differ from the actual
@@ -350,14 +346,14 @@ to the number of RRs in the update section.
 
 sub nscount {
 	my ( $self, @value ) = @_;
-	for (@value) { carp 'packet->header->nscount attribute is read-only' unless $warned++ }
+	for (@value) { $self->_warn('packet->header->nscount is read-only') }
 	return $$self->{count}[2] || scalar @{$$self->{authority}};
 }
 
 
 =head2 arcount, adcount
 
-    print "# of additional records: ", $packet->header->arcount, "\n";
+	print "# of additional records: ", $packet->header->arcount, "\n";
 
 Returns the number of records in the additional section of the packet
 which may, in the case of corrupt packets, differ from the actual
@@ -368,7 +364,7 @@ In dynamic update packets, this field is known as C<adcount>.
 
 sub arcount {
 	my ( $self, @value ) = @_;
-	for (@value) { carp 'packet->header->arcount attribute is read-only' unless $warned++ }
+	for (@value) { $self->_warn('packet->header->arcount is read-only') }
 	return $$self->{count}[3] || scalar @{$$self->{additional}};
 }
 
@@ -380,13 +376,12 @@ sub adcount { return &arcount; }
 
 =head1 EDNS Protocol Extensions
 
+=head2 do, co
 
-=head2 do
+	print "DNSSEC_OK flag was ", $packet->header->do ? "not" : "", "set\n";
+	$packet->header->do(1);
 
-    print "DNSSEC_OK flag was ", $packet->header->do ? "not" : "", "set\n";
-    $packet->header->do(1);
-
-Gets or sets the EDNS DNSSEC OK flag.
+Gets or sets the named EDNS flag.
 
 =cut
 
@@ -395,6 +390,11 @@ sub do {
 	return $self->_ednsflag( 0x8000, @value );
 }
 
+sub co {
+	my ( $self, @value ) = @_;
+	return $self->_ednsflag( 0x4000, @value );
+}
+
 
 =head2 Extended rcode
 
@@ -403,30 +403,26 @@ EDNS extended rcodes are handled transparently by $packet->header->rcode().
 
 =head2 UDP packet size
 
-    $udp_max = $packet->header->size;
-    $udp_max = $packet->edns->size;
+	$udp_max = $packet->edns->UDPsize;
 
 EDNS offers a mechanism to advertise the maximum UDP packet size
 which can be assembled by the local network stack.
 
-UDP size advertisement can be viewed as either a header extension or
-an EDNS feature.  Endless debate is avoided by supporting both views.
-
 =cut
 
-sub size {
+sub size {				## historical
 	my ( $self, @value ) = @_;
-	return $$self->edns->size(@value);
+	return $$self->edns->UDPsize(@value);
 }
 
 
 =head2 edns
 
-    $header  = $packet->header;
-    $version = $header->edns->version;
-    @options = $header->edns->options;
-    $option  = $header->edns->option(n);
-    $udp_max = $packet->edns->size;
+	$header  = $packet->header;
+	$version = $header->edns->version;
+	@options = $header->edns->options;
+	$option  = $header->edns->option(n);
+	$udp_max = $packet->edns->UDPsize;
 
 Auxiliary function which provides access to the EDNS protocol
 extension OPT RR.
@@ -464,6 +460,14 @@ sub _ednsflag {
 }
 
 
+my %warned;
+
+sub _warn {
+	my ( undef, @note ) = @_;
+	return carp "usage; @note" unless $warned{"@note"}++;
+}
+
+
 1;
 __END__
 
@@ -503,7 +507,7 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl> L<Net::DNS> L<Net::DNS::Packet> L<Net::DNS::RR::OPT>
-L<RFC1035(4.1.1)|https://tools.ietf.org/html/rfc1035>
+L<RFC1035(4.1.1)|https://iana.org/go/rfc1035#section-4.1.1>
 
 =cut
 
